// index.js
const express = require('express');
const ytSearch = require('yt-search');
const { exec } = require('child_process');
const fs = require('fs');
const path = require('path');
const cors = require('cors');
const app = express();
const port = 4286;

//use cors
app.use(cors());

// Middleware to parse JSON
app.use(express.json());

// Search endpoint using yt-search
app.get('/search', async (req, res) => {
    const query = req.query.q;

    if (!query) {
        return res.status(400).json({ error: 'Query parameter "q" is required.' });
    }

    try {
        const result = await ytSearch(query);
        const videos = result.videos.slice(0, 1000); // Get up to 1000 videos

        // Format videos for response
        const formattedVideos = videos.map(v => {
            const views = String(v.views).padStart(10, ' ');
            return {
                views: views,
                title: v.title,
                timestamp: v.timestamp,
                author: v.author.name,
                url: v.url
            };
        });

        res.json(formattedVideos);
    } catch (error) {
        res.status(500).json({ error: 'An error occurred while searching for videos.' });
    }
});

// Video download endpoint
app.get('/download', (req, res) => {
    const videoUrl = req.query.url;

    if (!videoUrl) {
        return res.status(400).json({ error: 'Query parameter "url" is required.' });
    }

    // Set the output file path
    var outputPath = path.join('/tmp/downloads', '%(title)s.%(ext)s');

    // Create /tmp/downloads directory if it doesn't exist
    fs.mkdirSync(path.join('/tmp/downloads'), { recursive: true });

    // Use yt-dlp to download the video
    exec(`yt-dlp -o "${outputPath}" "${videoUrl}"`, (error, stdout, stderr) => {
        if (error) {
            return res.status(500).json({ error: 'An error occurred while downloading the video.', details: stderr });
        }

        if (!filename) {
            return res.status(500).json({ error: 'Downloaded file not found.' });
        }

        // Full path of the downloaded file
        const downloadedFilePath = outputPath;

        // Check if the file exists and send it
        if (fs.existsSync(downloadedFilePath)) {
            res.download(downloadedFilePath, (err) => {
                if (err) {
                    return res.status(500).json({ error: 'Failed to send the file.', details: err.message });
                }

                // Optionally, delete the file after sending it
                fs.unlink(downloadedFilePath, (err) => {
                    if (err) {
                        console.error('Failed to delete the file:', err);
                    }
                });
            });
        } else {
            res.status(404).json({ error: 'Downloaded file not found.' });
        }
    });
});

app.listen(port, () => {
    console.log(`Server running at http://localhost:${port}`);
});

